local Sol = LibStub("Sol")

ItemPreview.constants = {
    MIN_ID       = 200000,         -- start id of the whole list
    MAX_ID       = 800000,         -- end id   ^
    MIN_EQUIP_ID = 200000,         -- start id of the equipment category
    MAX_EQUIP_ID = 250000,         -- end id   ^
    MIN_SKILL_ID = 490000,         -- start id of the skills category
    MAX_SKILL_ID = 495000,         -- end id   ^
    
    MIN_BUTTONS = 6,               -- minimum #buttons in items list 
    MAX_BUTTONS = 40,              -- maximum #buttons in items list 
    TOOLTIP_LINES = 40,            -- number of tooltip lines to read
 
    QUALITY_COLORS = {             -- hex colors for the different qualities/id types
        "ffffff", -- any
        "ffffff", -- white
        "00ff00", -- green
        "0072bc", -- blue
        "c805f8", -- purple
        "f68e56", -- salmon
        "a37d50", -- brown
        "a864a8", -- dark purple
        "f7941d", -- orange
        "fff466", -- yellow
        "ffff00", -- dark yellow
        "309166", -- dark green
        "fffff0", -- off-white
    },
}

local qualityColorNames = {         -- names for the different qualities/id types
    "Any quality",
    ITEM_QUALITY0_DESC or "White",
    ITEM_QUALITY1_DESC,
    ITEM_QUALITY2_DESC,
    ITEM_QUALITY3_DESC,
    ITEM_QUALITY4_DESC,
    ITEM_QUALITY5_DESC,
    "Item Mall",
    "Quest",
    "Self Skill",
    "Skill",
    "Passive Skill",
    "All others",
}

local presetNames = {               -- names for the ranges of IDs with known good values
    "All",
    "Equipment",
    "Skills",
    "Cards",
    "Recipes",
    "Runes",
}

local presetValues = {              -- ranges of IDs with known good values
    { min = ItemPreview.constants.MIN_ID, max = ItemPreview.constants.MAX_ID },
    { min = ItemPreview.constants.MIN_EQUIP_ID, max = ItemPreview.constants.MAX_EQUIP_ID },
    { min = 490000, max = 495000 },
    { min = 770000, max = 771000 },
    { min = 550000, max = 551200 },
    { min = 520001, max = 530000 }, -- 520001-530000
}

local sortModes = {
    "Sort by: ID",
    "Name",
    "Quality",
    "Level",
    "Update date",
}

local sortFunctions = {            -- comparison functions for sorting list data
    -- ID
    function(a, b) 
        return a < b
    end,
    -- Name,
    function(a, b) 
        return ItemPreview.itemList[a].text < ItemPreview.itemList[b].text
    end,
    -- Quality
    function(a, b)
        a = Sol.table.GetKey(ItemPreview.constants.QUALITY_COLORS, ItemPreview.itemList[a].color)
        b = Sol.table.GetKey(ItemPreview.constants.QUALITY_COLORS, ItemPreview.itemList[b].color)
        if not a then return false end
        if not b then return true end
        return a > b
    end,
    -- Level
    function(a, b)
        a = ItemPreview.itemList[a].lvl
        b = ItemPreview.itemList[b].lvl
        if not a then return false end
        if not b then return true end
        return a > b
    end,
    -- Date
    function(a, b)
        if a == b then return false end
        if not a then return false end
        if not b then return true end
        
        aDt = ItemPreview.itemList[a] and ItemPreview.itemList[a].date
        bDt = ItemPreview.itemList[b] and ItemPreview.itemList[b].date
        if not aDt and not bDt then return  end
        if not aDt then return false end
        if not bDt then return true end
        if aDt == bDt then return a < b end
    
        local aMonth, aDay, aYear = Sol.string.ParseDate(aDt)
        local bMonth, bDay, bYear = Sol.string.ParseDate(bDt)
        if not aYear and not bYear then return a < b end
        
        if not aYear then return false end
        if not bYear then return true end
        
        if aYear > bYear then return true end
        if aYear < bYear then return false end
        if aMonth > bMonth then return true end
        if aMonth < bMonth then return false end
        if aDay > bDay then return true end
        if aDay < bDay then return false end
        
        return a < b
    end,
}


local tooltips = {                  -- tooltips to display for the search screen elements
    -- main frame
    txtSearch   = "Enter search text. Available keywords include ",
    btnAdvanced = "Toggle advanced search settings",
    btnRefresh  = "Import item data that hasn't been saved yet. It should take a considerable amount of time the first time you do this, but significantly less afterwards. You may wish to do this after every patch",
    frmResize   = "Hold and drag to resize the ItemPreview window",
    ddSort      = "Sort the item list",
    
    -- advanced search frame
    ddRange     = "Determines the range of IDs to show",
    txtStartID  = "Start at this item ID",
    txtEndID    = "End at this item ID",
    txtMinLevel = "The minimum level of the item",
    txtMaxLevel = "The maximum level of the item",
    txtSlot     = "The inventory slot an item can go in (e.g., Head)",
    txtType     = "The equipment type of the item (e.g., Leather)",
    txtTooltip  = "Any other information in the tooltip (e.g., Quest Item)",
    btnSearch   = "Perform advanced search",
    chkIgnoreCase     = "Specifies whether to ignore case when searching",

    -- config frame
    sldUpdateSpeed    = "Determines how quickly refreshes and plus/tiers finish (and how laggy the game becomes while IP's working)",
    chkSaveTooltips   = "Enable this before importing to allow advanced search functionality",
    sldSearchDelay    = "When a key is typed in the main search window, IP waits a bit before performing the search. This specifies how many seconds to wait (0 for no wait)",
    chkShowMobs       = "Check to save which mobs drop what items and show this in the IP tooltip. You have to be the one to loot in order to save this data",
    chkShowZone       = "Check to save which zones an item can show up in and display this in the IP tooltip. You have to be the one to loot in order to save this data",
    btnForceRefresh   = "Force a complete refresh of all saved data. \n\nWARNING: this takes a considerable amount of time and should only be done if you believe the saved data has become stagnant",
    chkShowUpdated    = "Check to show in the tooltip the last time IP updated the item",
    
    -- plus frame
    txtPlus      = "The plus level of an item (i.e. enter 5 for Frost Crossbow +5; defaults to +6)",
    txtTier      = "The tiers to add to an item (i.e. enter 5 for a tier 8 Frost Crossbow; defaults to 0)", 

    txtDura      = "New Current Durability of the item (i.e. if you want to view OD)", 
    txtRarity    = "The number of rarity levels to add to an item (i.e. enter 2 for Brown Hailstone Scimitar, +2 above base 3)", 

    txtStatA     = "Complete name of the stat to add (i.e. Ability of the Temple VIII); not case-senstive",
    txtStatB     = "Complete name of the stat to add (i.e. Guard of the Temple VIII); not case-senstive",
    txtStatC     = "Complete name of the stat to add (i.e. Guardian of the Temple VIII); not case-senstive",
    txtStatD     = "Complete name of the stat to add (i.e. Defense of the Temple VIII); not case-senstive",
    txtStatE     = "Complete name of the stat to add (i.e. Power of the Temple VIII); not case-senstive",
    txtStatF     = "Complete name of the stat to add (i.e. Fire of the Temple VIII); not case-senstive",

    txtRune1     = "Complete name of the rune to add (i.e. Fatal I); not case-senstive",
    txtRune2     = "Complete name of the rune to add (i.e. Burst II); not case-senstive",
    txtRune3     = "Complete name of the rune to add (i.e. Hatred VI); not case-senstive",
    txtRune4     = "Complete name of the rune to add (i.e. Aggression VI); not case-senstive",

    chkHideStats = "Check if you don't want to add stats when +/tiering",
    chkHideRunes = "Check if you don't want to add runes when +/tiering",

    btnDoPlus    = "Begin the plus/tier operation",
    btnToggleSex = "Show a dummy of the opposite sex and then click items to try them on",
    
    -- skillup frame
    txtSkillLvl  = "Enter a level of this skill to preview",
}

-------------------------------------------------------------------------------
--  Config Init Functions                                                    --
-------------------------------------------------------------------------------
function ItemPreview:InitFrameNames()
    self.frames = {
        -- main frame
        frmMain     = ItemPreview_ItemListFrame,
        lstItems    = ItemPreview_ItemListFrameItemsList,
        lblTitle    = ItemPreview_ItemListFrameTitle,
        ddQuality   = ItemPreview_ItemListFrameQualityFilter,
        sbItems     = ItemPreview_ItemListFrameItemsListScrollBar,
        txtSearch   = ItemPreview_ItemListFrameSearchBox,
        btnAdvanced = ItemPreview_ItemListFrameAdvanced,
        btnRefresh  = ItemPreview_ItemListFrameRefreshList,
        frmResize   = ItemPreview_ItemListFrameResize,
        ddSort      = ItemPreview_ItemListFrameSort,
        
        frmSide     = ItemPreview_SidePanel,
        
        -- advanced search frame
        frmAdvanced = ItemPreview_AdvancedSearchFrame,
        txtStartID  = ItemPreview_AdvancedSearchFrameStartID,
        txtEndID    = ItemPreview_AdvancedSearchFrameEndID,
        ddRange     = ItemPreview_AdvancedSearchFrameRange,
        txtMinLevel = ItemPreview_AdvancedSearchFrameMinLevel,
        txtMaxLevel = ItemPreview_AdvancedSearchFrameMaxLevel,
        txtSlot     = ItemPreview_AdvancedSearchFrameSlot,
        txtType     = ItemPreview_AdvancedSearchFrameType,
        txtTooltip  = ItemPreview_AdvancedSearchFrameTooltip,
        btnSearch   = ItemPreview_AdvancedSearchFrameSearch,
        chkIgnoreCase = ItemPreview_AdvancedSearchFrameIgnoreCase,
        btnToggleSex  = ItemPreview_AdvancedSearchFrameToggleSex,
        
        -- config frame
        frmConfig       = ItemPreview_ConfigFrame,
        sldUpdateSpeed  = ItemPreviewConfig_NumberUpdateSpeed,
        sldSearchDelay  = ItemPreviewConfig_NumberSearchDelay,
        chkSaveTooltips = ItemPreviewConfig_ToggleSaveTooltips,
        btnForceRefresh = ItemPreview_ConfigFrameForceRefresh,  
        chkShowMobs     = ItemPreviewConfig_ToggleShowMobs,
        chkShowZone     = ItemPreviewConfig_ToggleShowZone,
        chkShowUpdated  = ItemPreviewConfig_ToggleShowUpdateTime,
        
        -- plus frame
        frmPlus         = ItemPreview_ItemPlusFrame,
        txtPlus         = ItemPreview_ItemPlusFramePlus,
        txtTier         = ItemPreview_ItemPlusFrameTier,

	txtDura      	= ItemPreview_ItemPlusFrameDura, 
	txtRarity    	= ItemPreview_ItemPlusFrameRarity, 

	txtStatA     	= ItemPreview_ItemPlusFrameStatA, 
	txtStatB     	= ItemPreview_ItemPlusFrameStatB, 
	txtStatC     	= ItemPreview_ItemPlusFrameStatC, 
	txtStatD     	= ItemPreview_ItemPlusFrameStatD, 
	txtStatE     	= ItemPreview_ItemPlusFrameStatE, 
	txtStatF     	= ItemPreview_ItemPlusFrameStatF, 

	txtRune1     	= ItemPreview_ItemPlusFrameRune1, 
	txtRune2     	= ItemPreview_ItemPlusFrameRune2, 
	txtRune3     	= ItemPreview_ItemPlusFrameRune3, 
	txtRune4     	= ItemPreview_ItemPlusFrameRune4, 

	chkHideStats	= ItemPreview_ItemPlusFrameHideStats,
	chkHideRunes	= ItemPreview_ItemPlusFrameHideRunes,

        btnDoPlus       = ItemPreview_ItemPlusFrameDoPlus,
        lblPlusWorking  = ItemPreview_ItemPlusFrameWorkingLabel,
        txtSkillLvl     = ItemPreview_ItemPlusFrameLevel,
        
        -- other
        tooltip         = ItemPreview_Tooltip,
	datatip		= ItemPreview_Datatip,
    }
    
    self.tabs = {
        self.frames.frmAdvanced,
        self.frames.frmPlus,
        self.frames.frmConfig,
    }
    self.frames.frmSide.tab = ItemPreview_SidePanelTabSearch
end

-- Initializes the IP windows and their subcomponents
function ItemPreview:LoadConfig()
    self:InitFrameNames()
    self:InitSearchFunctions()
    self:InitTooltipLines()
    self:InitFocus()
    
    Sol.config.RestoreFrameBounds(self.name, self.frames.tooltip)
    Sol.config.RestoreFrameBounds(self.name, self.frames.frmMain)
    
    -- Title
    if not self.itemList.count or self.itemList.count == 0 then
        self.frames.lblTitle:SetText("ItemPreview (0 items; please Import)")
    else
        self.frames.lblTitle:SetText("ItemPreview (" .. self.itemList.count .. " items)")
    end
    
    -- Button labels
    for i = 1, self.constants.MAX_BUTTONS do 
        local item = _G[self.frames.lstItems:GetName() .. "Item" .. i]
        item.label = _G[item:GetName() .. "Label"]
    end
    
    -- Quality colors
    local qualityColors = {}
    for i = 1, #self.constants.QUALITY_COLORS do
        qualityColors[i] = Sol.color.ColorText(qualityColorNames[i], self.constants.QUALITY_COLORS[i])
    end
    Sol.config.SetupDropdown(self.frames.ddQuality, qualityColors, 1, 80, self.SelectQuality)
    Sol.config.SetupDropdown(self.frames.ddSort, sortModes, 1, 80, self.SelectSortMode)
    Sol.config.SetupDropdown(self.frames.ddRange, presetNames, 1, 80, self.SelectRange)
    self.frames.sbItems:SetValueStepMode("INT")
    
    -- Update speed
    self:InitSlider(self.frames.sldUpdateSpeed, "Update Speed", 1, 300, "INT", self.settings.UpdateSpeed)
    self:InitSlider(self.frames.sldSearchDelay, "Search Delay", 0, 10, "FLOAT", self.settings.SearchDelay)
    
    -- Tooltips
    local keywords = ""
    for k, v in pairs(self.SearchFunctions) do
        keywords = keywords .. (k .. ": ")
    end
    tooltips.txtSearch = tooltips.txtSearch .. keywords
    for elem, tip in pairs(tooltips) do
        self.frames[elem].tooltip = tip
    end
    
    self.SelectRange(_, 1, _)
    self.SelectSortMode(_, self.settings.SortMode or 1, _)
    self:CheckListBounds()
end

-- Initialize a slider with the specified values
function ItemPreview:InitSlider(slider, nameText, min, max, stepMode, value)
    local sliderName = slider:GetName()
    slider:SetValueStepMode(stepMode)
    slider:SetMinMaxValues(min, max)
    slider:SetValue(value or min)
	_G[sliderName .. "Name"]:SetText(nameText)
	_G[sliderName .. "Value"]:SetText(value or min)
	_G[sliderName .. "Low"]:SetText(min)
	_G[sliderName .. "High"]:SetText(max)
end

-- Called when the quality dropdown changes
function ItemPreview.SelectQuality(dropdown, id, val)
    ItemPreview:AdvancedSearch()
end

-- Called when the type dropdown changes
function ItemPreview.SelectRange(dropdown, id, val)
    ItemPreview.frames.txtStartID:SetNumber(presetValues[id].min)
    ItemPreview.frames.txtEndID:SetNumber(presetValues[id].max)
    ItemPreview:AdvancedSearch()
end

function ItemPreview.SelectSortMode(dropdown, id, val)
    local self = ItemPreview
    self.settings.SortMode = id
    
    local sortFn = sortFunctions[id]
    self:Sort(sortFn)
end

-- Shows tooltips for frame elements
function ItemPreview:SearchElement_OnEnter(element, text)
    text = text or element.tooltip
    
    GameTooltip:SetOwner(element, "ANCHOR_BOTTOMRIGHT")
    GameTooltip:SetText(text)
    GameTooltip:Show()
end

-- Called when the IP window is hidden
function ItemPreview:OnHideListFrame()
    self.frames.btnAdvanced:SetText("More >>")
    self.frames.btnAdvanced:Enable()
    self.frames.frmSide:Hide()
end

-- Called when the IP window is shown
function ItemPreview:OnShowListFrame()
    self.SelectSortMode(_, self.settings.SortMode or 1, _)
    self:UpdateList()
end

-- Called when a slider's button is moved
function ItemPreview:OnValueChanged(slider)
    local value = Sol.math.Round(slider:GetValue(), 2)
	_G[slider:GetName().."Value"]:SetText(value)
    if (slider == self.frames.sldUpdateSpeed) then
        self.settings.UpdateSpeed = value
    elseif (slider == self.frames.sldSearchDelay) then
        self.settings.SearchDelay = value
    end
end

function ItemPreview:InitFocus()
    local focusTable = {
        self.frames.txtSearch,
        self.frames.txtStartID,
        self.frames.txtEndID,
        self.frames.txtTooltip,
        self.frames.txtMinLevel,
        self.frames.txtMaxLevel,
        self.frames.txtSlot,
        self.frames.txtType,
        self.frames.txtPlus,
        self.frames.txtTier,

        self.frames.txtDura,
        self.frames.txtRarity,

        self.frames.txtStatA,
        self.frames.txtStatB,
        self.frames.txtStatC,
        self.frames.txtStatD,
        self.frames.txtStatE,
        self.frames.txtStatF,

        self.frames.txtRune1,
        self.frames.txtRune2,
        self.frames.txtRune3,
        self.frames.txtRune4,

	self.frames.chkHideStats,
	self.frames.chkHideRunes,

        self.frames.txtSkillLvl,
    }
    
    local prev, first
    for i, elem in ipairs(focusTable) do
        if not first then first = elem end
        if prev then
            prev.nextFocus = elem
            elem.previousFocus = prev
        end
        prev = elem
    end
    if first and prev then
        -- prev == last
        prev.nextFocus = first
        first.previousFocus = prev
    end
end

function ItemPreview:ClearFocus()
    for name, elem in pairs(self.frames) do
        if name ~= "txtSearch" and Sol.string.StartsWith(name, "txt") then
            elem:ClearFocus()
        end
    end
end

function ItemPreview:ShiftFocus(textbox)
    if IsShiftKeyDown() then
        local current = textbox.previousFocus
        while current and current ~= textbox and not current:GetParent():IsVisible() do
            current = current.previousFocus
        end
        if current then
            current:SetFocus()
        else
            textbox:ClearFocus()
        end
    else
        local current = textbox.nextFocus
        while current and current ~= textbox and not current:GetParent():IsVisible() do
            current = current.nextFocus
        end
        if current then
            current:SetFocus()
        else
            textbox:ClearFocus()
        end						
    end
end

-- Show or hide the advanced search frame
function ItemPreview:ToggleAdvancedSearch()
    local scale = GetUIScale()
    if self.frames.frmSide:IsVisible() then
        local w, h = self.frames.frmSide:GetSize()
        local x, y = self.frames.frmMain:GetPos()
        x = x + self.frames.frmMain:GetSize() * scale
        
        self.frames.btnAdvanced:Disable()
        Sol.animation.Move(self.name, self.frames.frmSide, x - w, y, nil, 0.2, nil, function() 
            self.frames.frmSide:Hide() 
            self.frames.btnAdvanced:SetText("More >>")
            self.frames.btnAdvanced:Enable()
        end)
    else
        local w, h = self.frames.frmSide:GetSize()
        local x, y = self.frames.frmMain:GetPos()
        x = x + self.frames.frmMain:GetSize() * scale
        self.frames.frmSide:SetPos(x - w, y)
        
        self.frames.frmSide:Show()
        self.frames.btnAdvanced:Disable()
        Sol.animation.Move(self.name, self.frames.frmSide, x - 3, y, nil, 0.2, nil, function()
            self.frames.btnAdvanced:SetText("Less <<")
            self.frames.btnAdvanced:Enable()
        end)
    end
end

function ItemPreview:OnTabClicked(tab, id) 
    SkillTab_SetActiveState(self.frames.frmSide.tab, false)
    SkillTab_SetActiveState(tab, true)
    self.frames.frmSide.tab = tab
    for i = 1, #self.tabs do
        if i ~= id then
            self.tabs[i]:Hide()
        else
            self.tabs[i]:Show()
        end
    end
end
