SLASH_TESTLINK1 = "/testlink"
SlashCmdList["TESTLINK"] = function (editBox, msg)
	if msg:match("^(%x+)%s+(%x+)%s+(%x+)%s+(%x+)%s+(%x+)%s+(%x+)%s+(%x+)%s+(%x+)%s+(%x+)%s+(%x+)%s+(%x+)") then
		local hash = CalculateItemLinkHash(msg)
		ChatFrame1:AddMessage(string.format("Hash: %x", hash))  
		ChatFrame1:AddMessage(string.format("|Hitem:%s %x|h[Item Test]|h", msg, hash)) 
	else
		ChatFrame1:AddMessage("Incorrect format!") 
	end
end

local ip_debug = false

SLASH_IPDEBUG1 = "/ipdebug"
SlashCmdList["IPDEBUG"] = function (editBox, msg)
	if ip_debug == false then
		ip_debug = true
		DEFAULT_CHAT_FRAME:AddMessage("Debug messages: on")
	else
		ip_debug = false
		DEFAULT_CHAT_FRAME:AddMessage("Debug messages: off")
	end
end

--[[ Item link data --------------------------------------------------------------------------------

|Hitem:<itemid> <bind> <unknown&runePlus&tier&max dura> <statAB>x3 <runes>x4 <current dura> <hash>|h|cff<color>[<name>]|r|h

    Ex: |Hitem:33E7E 2 2c8c1064 0 0 0 0 0 0 0 2EE0 <hash>|h|cffc805f8[Yawaka Guard]|r|h
    Yawaka Guard, +12, T9, 4 empty rune slots, Dura: 120/100

itemid : length 5

    ItemID in hex (Ex: Yawaka Guard, ItemID: 212606, Hex: 33E7E)

bind : length 1-3

    1=nonbindable
    2=bound
    3=boe
    10=Equipment Protection (equipment locked?)
    30=Equipment Protection
    50=Equipment Protection
    70=Equipment Protection
    90=Equipment Protection
    100=prevent hijack (don't drop on PK death)
    201=seen when linking an item from craft frame (hides bind status and dura on the tooltip)
    400=set skill extracted
    500=set skill extracted and prevent hijack

unknown : length 0-2

    unknown, but it seems to be used to differentiate two identical items in the backpack. linking two identical items will make
    the same link but if you get the links (GetBagItemLink(invIndex)) of each item they will be different.
    Leaving it off doesn't affect any of the vital stats.

    Ex: 2 identical Fine Leather Belts
    1st: 36de9 1 a64 0 0 0 0 0 0 0 2710 5a6f
    2nd: 36de9 1 2f00a64 0 0 0 0 0 0 0 2710 d13e

    1st +1: 36de9 1 10a64 0 0 0 0 0 0 0 2710 da7a
    2nd +1: 36de9 1 2e010a64 0 0 0 0 0 0 0 2710 61d4

runePlus : length 0-2

    empty rune slots & plus

    00/20/40/60/80h = 0-4 empty rune slots
    00h-1Fh = + 0-31

    41 = 2 empty rune slots, +1
    8C = 4 empty rune slots, +12

    4 x 20h = 80h
    +12 = 0Ch
    80 + 0C = 8Ch

    90 = 4 empty rune slots, +16

    4 x 20h = 80h
    +16 = 10h
    80h + 10h = 90h

    Ex: 9Fh = 4 empty rune slots, +31
    Ex: 70h = 3 empty rune slots, +16
    Ex: 00h = no empty rune slots, +0
    Ex: 26h = 1 empty rune slot, +6

    "Rune: x/y" on the tooltip
    x = number of rune slots that are filled (up to 4, since there are only 4 spaces in the for runes stats)
    y = <# empty rune slots> + x

tier : length 1-2

    0A = not tiered, + to default tier (0Bh-1Fh = 1-21)
    2A = +1 rarity level, base tier
    4A = +2 rarity level, base tier
    + rarity level is seen in crafted items

    Ex: Brown Hailstone at T9 (rarity level 5 i.e. +2 above base of 3) = 50h
    4Ah + 06h = 50h

maxdura : length 2

    this is the visible max dura, straight Dec to Hex
    Ex: 100 dura = 64h

stats : length (4x2) x3

    4 each, 2 combine into an 8-hexadecimal digit string, 3 strings separated by spaces, or 0s if no stats there.

    Tooltip / hex string - Stat Order
    statB
    statA
    statD
    statC
    statF
    statE

    hex string
    <statA/statB> <statC/statD> <statE/statF>

runes : length 5 x4

    5 per rune, 4 seperated by spaces, 0s if no rune
    Ex: Fatal I, ItemID: 520761, Hex: 7F239

current dura : length 4

    this is current dura. <current dura>*100 converted to hex
    Ex: 8000 [1F40h in hex] for 80 current dura
    Ex: 10000 [2710h in hex] for 100 current dura
    Ex: 12000 [2EE0h in hex] for 120 current dura

hash : length 4

    Search for 'Runes of Magic item link hash calculation code' in 'ItemPreview/Lua/Main.lua' to view the code.
    Thanks to 'ohos' for getting the ball rolling and Valacar (Duppy of Osha-US) for completing the code.
    thread that started it all: https://forum.runesofmagic.com/showthread.php?p=2368150#post2368150
    code from: https://forum.runesofmagic.com/showthread.php?t=280594

----------------------------------------------------------------------------------]]

local VERSION = "v1.17"  
local Sol = LibStub("Sol")

ItemPreview = {}

ItemPreview.DefaultSettings = {
    UpdateSpeed = 10,
    UpdateInterval = 30,
    SaveTooltips = true,
    ScrollBy = 1,
    IgnoreCase = true,
    SearchDelay = 0,
    ShowMobs = true,
    ShowZone = true,
    ShowUpdateTime = true,
    ResizeTime = 0.5,
    SortMode = 1,
}

-- Stats  510001-520000
-- Runes  520001-530000
-- Titles 530001-540000
-- 
local firstStatID = 510001
local lastStatID = 520000
local firstRuneID =  520001
local lastRuneID = 530000

local isWorking = nil
local plusData = nil
local plusName = nil
local plusHash = 0

function ItemPreview:OnLoad(frame)
    frame:RegisterEvent("VARIABLES_LOADED")
    frame:RegisterEvent("BOOTY_SHOW")
    self.name = "ItemPreview"
    self.tag = "IP"
end

function ItemPreview:OnEvent(frame, _event, _arg1, _arg2)
	if ( _event == "VARIABLES_LOADED" ) then
        Sol.config.CheckSettings("ItemPreview", ItemPreview.DefaultSettings)
        self.settings = ItemPreview_Settings
        
        SaveVariables("ItemPreview_ListData")
        if not ItemPreview_ListData then
            self:ResetListData()
        end
        self.itemList = ItemPreview_ListData
    
        Sol.config.CreateSlashCommand("item", self.ShowItem)
        Sol.config.CreateSlashCommand("itemid", self.ShowID)
        Sol.config.CreateSlashCommand("skill", self.ShowSkill)
        Sol.config.CreateSlashCommand("plus", self.Plus)
        Sol.config.CreateSlashCommand("itempreview", self.PrintUsage)
        Sol.config.CreateSlashCommand("ipexample", self.PrintExamples)
        Sol.config.CreateSlashToHandleConfig(self.name, ItemPreview_ItemListFrame, "ip", 0.5, Sol.constants.FADE)
        
        self:RegisterWithAddonManager()
        self:LoadConfig()
    
    elseif ( _event == "BOOTY_SHOW" ) then
        if self.settings.ShowZone or self.settings.ShowMobs then
            self:OnLootWindowShown()
        end
    end
end

function ItemPreview:RegisterWithAddonManager() 
    local commands = "/ip, /item, /plus, /skill, /itemid, /ipexample"
    if AddonManager then
        local addon = {
            name = self.name, 
            description = "Lets you preview items you don't have, plus and tier items to " ..
                          "see what the stats will be, and preview skills at different levels", 
            category = "Information",
            version = VERSION,
            author = "Alleris",
            slashCommands = commands,
            icon = "Interface/Addons/ItemPreview/Textures/ItemPreviewIcon",
            miniButton = ItemPreview_MiniButton,
            configFrame = ItemPreview_ItemListFrame,
        }
        
        if AddonManager.RegisterAddonTable then
            AddonManager.RegisterAddonTable(addon)
        else
            AddonManager.RegisterAddon(addon.name, addon.description, addon.icon, addon.category, 
                addon.configFrame, addon.slashCommands, addon.miniButton, addon.onClickScript)
        end
    else
        Sol.io.Print("ItemPreview loaded: " .. commands)
    end
end

-- Called when the player opens a loot window
function ItemPreview:OnLootWindowShown()
    local mob = UnitName("target")
    for i = 1, GetBootyNumItems() do
        local link = GetBootyItemLink(i)
        if link then 
            local hexid = link:match("^|H.-:([0-9a-f]+)")
            local id = Sol.math.HexToDec(hexid)
            if id and mob and self.settings.ShowMobs then
                self:UpdateMobForItem(id, mob)
            end
            if id and self.settings.ShowZone then
                self:UpdateZoneForItem(id, GetZoneID() % 1000 )
            end
        end
    end
end

function ItemPreview:ToggleSex()
    if UnitSex("player") == 1 then
        ItemPreviewFrameModel:SetDummyMale()
    else
        ItemPreviewFrameModel:SetDummyFemale()
    end
end

function ItemPreview.ShowID(_, link)
    if not link then return end
    
    local id = link:match("^|H.-:([0-9a-f]+)")
    if id then 
        Sol.io.Print(string.format("%s ID: %d (0x%s)", link, Sol.math.HexToDec(id), id))
    end
end

function ItemPreview.ShowSkill(_, msg)
    local self = ItemPreview
    if not msg or #msg < 2 then 
        self:PrintUsage()
        return
    end
    
    local id, lvl = msg:match("^(%d+) ?(%d*)$")
    if not id then 
        id, lvl = msg:match("^.-(%d+).-|r|h.-(%d+)")
    end
    if not id then
        id = msg:match("(%d+)")
    end
    self:ShowSkillUp(id, lvl)
end

function ItemPreview:ShowSkillUp(id, lvl)
    if not id then return end
    if not lvl or lvl == "" then
        lvl = "55"
    end
    
    local link = string.format("|Hskill:%s %s|h|h", id, lvl)
    local text, hexColor = self:GetItemLinkDataFromString(link)
    if text then
        if not hexColor then
            hexColor = "ffffff"
        end
        link = string.format("|Hskill:%s %s|h|cff%s[%s]|r|h", id, lvl, hexColor, text)
        
        self:ShowTooltip(link, true)
    else
        self:ShowTooltip(link)
    end
end

function ItemPreview.Plus(_, msg)
    local self = ItemPreview
    if not msg or #msg < 2 then
        self:PrintUsage()
        return
    end
    
    local numToPlus = msg:match("^%+(%d+)")
    if numToPlus then
        msg = msg:sub(numToPlus:len()+3)
        numToPlus = tonumber(numToPlus)
    else
        numToPlus = 6
    end
    local numToTier = msg:match("^t(%d+)")
    if numToTier then
	msg = msg:sub(numToTier:len()+3)
        numToTier = tonumber(numToTier)
    end
    local dura = msg:match("^d(%d+)")
    if dura then
        msg = msg:sub(dura:len() + 3)
        dura = tonumber(dura)
    end
    local rarity = msg:match("^r(%d+)")
    if rarity then
        msg = msg:sub(rarity:len() + 3)
        rarity = tonumber(rarity)
    end
    if self.DoPlus(numToPlus, numToTier, msg, dura, rarity) then
	for n=1,10 do 
		getglobal("ChatFrame"..n):AddMessage("|cffffffff".."Working. Please wait...".."|r")
	end
    end
end    

function ItemPreview.DoPlus(numToPlus, numToTier, msg, dura, rarity, statAB, statCD, statEF, rune1, rune2, rune3, rune4, hidestats, hiderunes)
	if not msg or (not numToPlus and not numToTier) then return false end

	numToPlus = tonumber(numToPlus)
	numToTier = tonumber(numToTier)

	local self = ItemPreview    

	local rawdata = msg:match("^|Hitem:(.+)(|h.+)$")
	local ItemID_bind, UnknownRuneplusTierMaxdura, Stats_Runes, Dura, hash, name = msg:match("^(|Hitem:%x+ %x+ )(%x+)( %x+ %x+ %x+ %x+ %x+ %x+ %x+ )(%x+) (%x+)(|h.+)$")
	local AB, CD, EF, r1, r2, r3, r4 = nil
	if Stats_Runes then
		AB, CD, EF, r1, r2, r3, r4 = Stats_Runes:match("^ (%x+) (%x+) (%x+) (%x+) (%x+) (%x+) (%x+) $")
	end

	if ip_debug == true then
		Sol.io.Print("--------------------")
		Sol.io.Print(rawdata.." : rawdata")
		if ItemID_bind then Sol.io.Print(ItemID_bind.." : ItemID_bind") end
		if UnknownPlusTierMaxdura then Sol.io.Print(UnknownRuneplusTierMaxdura.." : UnknownPlusTierMaxdura") end
		if Stats_Runes then Sol.io.Print(Stats_Runes.." : Stats_Runes") end
		if Dura then Sol.io.Print(Dura.." : Dura") end
		if hash then Sol.io.Print(hash.." : hash") end
		if name then Sol.io.Print(name.." : name") end
		Sol.io.Print("--------------------")
	end
	local unknown, runeplus, tier, maxdura = "", "", "", ""
	if not ItemID_bind then
		ItemID_bind, name = msg:match("^(|Hitem:%x+)(.+)$")
		if not ItemID_bind then
			local hex = string.format("%x", msg)
			ItemID_bind, name = self:MakeItemLinkFromID(hex):match("^(|Hitem:%x+)(.+)$")
		end
		ItemID_bind = ItemID_bind.. " 1 "

		tier = "a"
		maxdura = "64"
		--Stats_Runes = " 0 0 0 0 0 0 0 "
		if not dura then
			Dura = "2EE0"
		else
			Dura = string.format("%x", dura * 100)
		end
	else
		if ip_debug == true then Sol.io.Print(UnknownRuneplusTierMaxdura.." : Splitting UnknownRuneplusTierMaxdura") end
		
		if UnknownRuneplusTierMaxdura:match("%x%x%x%x%x%x%x%x") then
			unknown, runeplus, tier, maxdura = UnknownRuneplusTierMaxdura:match("(%x%x)(%x%x)(%x%x)(%x%x)")
			if ip_debug == true then Sol.io.Print(UnknownRuneplusTierMaxdura.." : (%x%x)(%x%x)(%x%x)(%x%x)") end
		elseif UnknownRuneplusTierMaxdura:match("%x%x%x%x%x%x%x") then
			unknown, runeplus, tier, maxdura = UnknownRuneplusTierMaxdura:match("(%x)(%x%x)(%x%x)(%x%x)")
			if ip_debug == true then Sol.io.Print(UnknownRuneplusTierMaxdura.." : (%x)(%x%x)(%x%x)(%x%x)") end
		elseif UnknownRuneplusTierMaxdura:match("%x%x%x%x%x%x") then
			runeplus, tier, maxdura = UnknownRuneplusTierMaxdura:match("(%x%x)(%x%x)(%x%x)")
			if ip_debug == true then Sol.io.Print(UnknownRuneplusTierMaxdura.." : (%x%x)(%x%x)(%x%x)") end
		elseif UnknownRuneplusTierMaxdura:match("%x%x%x%x%x") then
			runeplus, tier, maxdura = UnknownRuneplusTierMaxdura:match("(%x)(%x%x)(%x%x)")
			if ip_debug == true then Sol.io.Print(UnknownRuneplusTierMaxdura.." : (%x)(%x%x)(%x%x)") end
		elseif UnknownRuneplusTierMaxdura:match("%x%x%x%x") then
			tier, maxdura = UnknownRuneplusTierMaxdura:match("(%x%x)(%x%x)")
			if ip_debug == true then Sol.io.Print(UnknownRuneplusTierMaxdura.." : (%x%x)(%x%x)") end
		elseif UnknownRuneplusTierMaxdura:match("%x%x%x") then
			tier, maxdura = UnknownRuneplusTierMaxdura:match("(%x)(%x%x)")
			if ip_debug == true then Sol.io.Print(UnknownRuneplusTierMaxdura.." : (%x)(%x%x)") end
		end
	end

	if ip_debug == true then
		Sol.io.Print("----------")
		Sol.io.Print(ItemID_bind.." : ItemID_bind")
		Sol.io.Print(unknown.." : unknown")
		Sol.io.Print(runeplus.." : runeplus")
		Sol.io.Print(tier.." : tier")
		Sol.io.Print(maxdura.." : maxdura")
		if name then Sol.io.Print(name.." : name") end
	end

	local temptier = math.floor( tonumber(tier, 16) /32 ) * 32

	local RarityTable = {	[0] = "ffffff", [1] = "00ff00", [2] = "0072bc", [3] = "c805f8", [4] = "f68e56", [5] = "a37d50", }
	if rarity then
		local currentcolor = name:match("cff(.+)%[")
		local currentrarity, newrarity = nil
		for i = 0, 5 do
			if currentcolor == RarityTable[i] then
				currentrarity = i
			end
		end
		if currentrarity + rarity <= 5 then
			newrarity = currentrarity + rarity
			name = name:gsub("cff"..RarityTable[currentrarity],"cff"..RarityTable[newrarity])
			temptier = temptier + (rarity * 32)
		end
	end


	if numToTier then
		if runeplus ~= "" or unknown ~= "" or numToPlus > 0 then
			tier = string.format("%02x", temptier + 10 + numToTier)
		else
			tier = string.format("%x", temptier + 10 + numToTier)
		end
		if ip_debug == true then Sol.io.Print(tier.." : 3a numToTier") end
	elseif not numToTier then
		if runeplus ~= "" or unknown ~= "" or numToPlus > 0 then
			tier = string.format("%02x", temptier + 10)
		else
			tier = string.format("%x", temptier + 10)
		end
		if ip_debug == true then Sol.io.Print(tier.." : 3b not numToTier") end
	end

	if ip_debug == true then Sol.io.Print(numToPlus.." : numToPlus") end

	local currentrune, currentplus
	if runeplus ~= "" then 
		currentrune = math.floor(tonumber(runeplus,16)/tonumber("20",16))
		currentplus =  tonumber(runeplus,16) % tonumber("20",16)
	else
		currentrune = 0
		currentplus = 0
	end

	if numToPlus == nil then
		runeplus = currentrune * 32 + currentplus
		if unknown ~= "" then 
			runeplus = string.format("%02x", runeplus)
		else
			if runeplus > 0 then
				runeplus = string.format("%x", runeplus)
			end
		end
	else
		runeplus = currentrune * 32 + numToPlus
		if unknown ~= "" then 
			runeplus = string.format("%02x", runeplus)
		else
			if runeplus > 0 then
				runeplus = string.format("%x", runeplus)
			end
		end
	end

	if dura then 
		Dura = string.format("%04x", dura*100)
	end

	if statAB == nil and AB == nil then
		statAB = "0"
	elseif statAB == nil and AB ~= nil then
		statAB = AB
	end
	if statCD == nil and CD == nil then
		statCD = "0"
	elseif statCD == nil and CD ~= nil then
		statCD = CD
	end
	if statEF == nil and EF == nil then
		statEF = "0"
	elseif statEF == nil and EF ~= nil then
		statEF = EF
	end

	if rune1 == nil and r1 == nil then
		rune1 = 0
	elseif rune1 == nil and r1 ~= nil then
		rune1 = r1
	end
	if rune2 == nil and r2 == nil then
		rune2 = 0
	elseif rune2 == nil and r2 ~= nil then
		rune2 = r2
	end
	if rune3 == nil and r3 == nil then
		rune3 = 0
	elseif rune3 == nil and r3 ~= nil then
		rune3 = r3
	end
	if rune4 == nil and r4 == nil then
		rune4 = 0
	elseif rune4 == nil and r4 ~= nil then
		rune4 = r4
	end

	if hidestats then
		statAB, statCD, statEF = "0","0","0"
	end
	if hiderunes then
		rune1, rune2, rune3, rune4 = "0","0","0","0"
	end

	Stats_Runes = string.format(" %s %s %s %s %s %s %s ", statAB, statCD, statEF, rune1, rune2, rune3, rune4)

	plusData = ItemID_bind .. unknown .. runeplus .. tier .. maxdura .. Stats_Runes .. Dura .. " "

	if ip_debug == true then 
		Sol.io.Print(ItemID_bind.." : ItemID_bind")
		Sol.io.Print(unknown.." : unknown")
		Sol.io.Print(runeplus.." : runeplus")
		Sol.io.Print(tier.." : tier")
		Sol.io.Print(maxdura.." : maxdura")
		Sol.io.Print(Stats_Runes.." : Stats_Runes")
		Sol.io.Print(Dura.." : Dura")
		Sol.io.Print(plusData.." : plusData")
	end

	plusName = name
	plusHash = 1
	isWorking = true
	return true
end

function ItemPreview.ShowItem(_, msg)
    local self = ItemPreview
    if not msg or #msg < 2 then
        self:PrintUsage()
        return
    end
    
    local trimmed = Sol.string.Trim(msg)    
    local startID = tonumber(trimmed:match("(%d+)"))
    if not startID then return end
    
    local link = self:MakeItemLinkFromID(string.format("%x", startID))
    self:ShowLink(link, true)
end

function ItemPreview:ShowLink(link, ignoreChatFrame)
    ItemPreviewFrame_SetItemLink(ItemPreviewFrame, link)
    self:ShowTooltip(link, ignoreChatFrame)
end

function ItemPreview:ShowSkillFromDetails()
    local level = self.frames.txtSkillLvl:GetNumber()
    local link = self.frames.lstItems.selectedButton.label:GetText()
    local id = Sol.math.HexToDec(link:match("|H.-:([0-9a-f]+)"))
    self:ShowSkillUp(id, level)
end

function ItemPreview:ModNameToHex(modName)
	for i = firstStatID, lastStatID do
		local txt=TEXT("Sys"..i.."_name")
		if modName:lower() == txt:lower() then
			return string.match(string.format("%x", i),"^7(%x+)")
		end
	end
	return ""
end

function ItemPreview:ModNameToHexForManaStone(modName)
	for i = firstStatID, lastStatID do
		local txt=TEXT("Sys"..i.."_name")
		if modName:lower() == txt:lower() then
			return string.format("%04x", i-500000)
		end
	end
	return ""
end

function ItemPreview:RuneNameToHex(runeName)
	for i = lastRuneID, firstRuneID, -1 do
		local txt = TEXT("Sys"..i.."_name")
		if runeName:lower() == txt:lower() then
			return string.format("%x",i)
		end
	end
	return nil
end

function ItemPreview:DoPlusFromDetails()
	local plus = self.frames.txtPlus:GetNumber()
	local tier = self.frames.txtTier:GetNumber()
	local level = self.frames.txtSkillLvl:GetNumber()
	local link = self.frames.lstItems.selectedButton.label:GetText()

	local dura = self.frames.txtDura:GetNumber()
	local rarity = self.frames.txtRarity:GetNumber()

	if dura == 0 then dura = nil end
	if rarity == 0 then rarity = nil end

	local statA = self.frames.txtStatA:GetText()
	local statB = self.frames.txtStatB:GetText()
	local statC = self.frames.txtStatC:GetText()
	local statD = self.frames.txtStatD:GetText()
	local statE = self.frames.txtStatE:GetText()
	local statF = self.frames.txtStatF:GetText()

	local statAB, statCD, statEF = nil

	local id = Sol.math.HexToDec(link:match("|H.-:([0-9a-f]+)"))
	if id >= 202840 and id<= 202859 then
		statAB = self:ModNameToHexForManaStone(statB)..self:ModNameToHexForManaStone(statA)
		statCD = self:ModNameToHexForManaStone(statD)..self:ModNameToHexForManaStone(statC)
		statEF = self:ModNameToHexForManaStone(statF)..self:ModNameToHexForManaStone(statE)
	else
		statAB = self:ModNameToHex(statB)..self:ModNameToHex(statA)
		statCD = self:ModNameToHex(statD)..self:ModNameToHex(statC)
		statEF = self:ModNameToHex(statF)..self:ModNameToHex(statE)
	end

	if statAB == "" then statAB = nil end
	if statCD == "" then statCD = nil end
	if statEF == "" then statEF = nil end

	local rune1 = self:RuneNameToHex(self.frames.txtRune1:GetText())
	local rune2 = self:RuneNameToHex(self.frames.txtRune2:GetText())
	local rune3 = self:RuneNameToHex(self.frames.txtRune3:GetText())
	local rune4 = self:RuneNameToHex(self.frames.txtRune4:GetText())

	local hidestats, hiderunes = nil
	if self.frames.chkHideStats:IsChecked() then
		hidestats = true
	else
		hidestats = false
	end
	if self.frames.chkHideRunes:IsChecked() then
		hiderunes = true
	else
		hiderunes = false
	end

	if id >= 490000 and id <= 495000 then
		self:ShowSkillUp(id, level)
	elseif self.DoPlus(plus, tier, link, dura, rarity, statAB, statCD, statEF, rune1, rune2, rune3, rune4, hidestats, hiderunes) then
		self.frames.lblPlusWorking:SetText("Working... Please wait")
	else
		self.frames.lblPlusWorking:SetText("Unable to perform requested action")
	end
end

function ItemPreview:ShowTooltip(link, ignoreChatFrame)
    if link then
        self.frames.tooltip:SetHyperLink(link)
        self.frames.tooltip:Show()
        if IsShiftKeyDown() or ignoreChatFrame then
            self:PrintLink(link, ignoreChatFrame)
        end
    else
        self.frames.tooltip:Hide()
    end
end

function ItemPreview:PrintLink(link, ignoreChatFrame)
	if not link:match("%["..self.tag) then
		if link:match("%["..self.name) then
			link = string.gsub(link, "%["..self.name..": ", string.format("[%s: ", self.tag))
		else
			link = string.gsub(link, "%[", string.format("[%s: ", self.tag))
		end
	end
    if not ignoreChatFrame and ChatFrameEditBox:IsVisible() then
        ChatFrameEditBox:InsertText(link)
    else
	for n=1,10 do 
		getglobal("ChatFrame"..n):AddMessage(link)
	end
    end    
end

function ItemPreview:MakeItemLink(hexID, text, hexColor)
    if text then
        if not hexColor then
            hexColor = "ffffff"
        end
        return string.format("|Hitem:%s|h|cff%s[%s]|r|h", hexID, hexColor, text)
    end
end

function ItemPreview:MakeItemLinkFromID(hexID)
    local text, color = self:GetItemLinkDataFromID(hexID)
    return self:MakeItemLink(hexID, text, color)
end

function ItemPreview:GetItemLinkDataFromString(str)
    self.frames.datatip:SetText("")
    self.frames.datatip:SetHyperLink(str)
	GameTooltip1:Hide()
	GameTooltip2:Hide()
    self.frames.datatip:Hide()
    local text = ItemPreview_DatatipTextLeft1:GetText()
    local color = Sol.color.ColorValuesToHexColor(ItemPreview_DatatipTextLeft1:GetColor()):lower()
    if text == "" then text = nil end
    return text, color
end

function ItemPreview:GetItemLinkDataFromID(hexID)
    return self:GetItemLinkDataFromString("|Hitem:" .. hexID .. "|h|h")
end

function ItemPreview:PrintUsage()
    Sol.io.Print("ItemPreview usage: ")
    
    Sol.io.Print("/item id")
    Sol.io.Print("Previews the item with the specified ID")
    Sol.io.Print("-  id: an item ID, can be found in the URL of the item on RoM database websites")
    Sol.io.Print("")
    
    Sol.io.Print("/itemid link")
    Sol.io.Print("Get the id of the specified item")
    Sol.io.Print("-  link: an item link")
    Sol.io.Print("")
    
    Sol.io.Print("/skill id [level]")
    Sol.io.Print("Previews the skill with the specified ID")
    Sol.io.Print("-  id: a skill ID, can be found in the URL of the item on RoM database websites")
    Sol.io.Print("-  level: optional - what level of the skill to preview")
    Sol.io.Print("")
    
    Sol.io.Print("/skill link")
    Sol.io.Print("Previews the specified skill at level 50")
    Sol.io.Print("-  link: a skill link, shift+click a skill to insert it")
    Sol.io.Print("")
    
    Sol.io.Print("/plus [+#] [t#] id")
    Sol.io.Print("/plus [+#] [t#] link")
    Sol.io.Print("Previews a plussed/tiered version of the item (may take a few seconds)")
    Sol.io.Print("-  +#: optional - how much to plus the item (default 6)")
    Sol.io.Print("-  t#: optional - how much to tier up the item (defaults to 0)")
    Sol.io.Print("-  id: an item ID, can be found in the URL of the item on RoM database websites")
    Sol.io.Print("-  link: an item link, shift+click an item to insert it")
    Sol.io.Print("")
    
    Sol.io.Print("/ipexample")
    Sol.io.Print("Shows examples of each of these to help you understand how to use it")
end

function ItemPreview:PrintExamples()
    local itemLink = "|cff0072bc[Frost Crossbow]|r"
    local spellLink = "|cff8080ff[Flame+35]|r"
    
    Sol.io.Print("ItemPreview Examples:")
    
    Sol.io.Print("/item 211549")
    Sol.io.Print("Previews the Frost Crossbow (id from http://www.runesdb.com/item.aspx?id=211549)")
    Sol.io.Print("/plus 211549")
    Sol.io.Print("Previews Frost Crossbow+6 ")
    Sol.io.Print("/plus t4 211549")
    Sol.io.Print("Previews Frost Crossbow+6, tier7 (4 tiers above default) ")
    Sol.io.Print("/plus +5 t4 211549")
    Sol.io.Print("Previews Frost Crossbow+5, tier7")
    Sol.io.Print("|cffffffff/plus " .. itemLink .. "|r")
    Sol.io.Print("Previews Frost Crossbow+6, with all stats of the linked item")
    Sol.io.Print("|cffffffff/plus +5 " .. itemLink .. "|r")
    Sol.io.Print("Previews Frost Crossbow+5, with all stats of the linked item")
    Sol.io.Print("|cffffffff/itemid " .. itemLink .. "|r")
    Sol.io.Print("Prints 211549")
    
    Sol.io.Print("/skill 491150")
    Sol.io.Print("Previews Flame at lvl50 (id from http://www.runesdb.com/spell.aspx?id=491150)")
    Sol.io.Print("/skill 491150 52")
    Sol.io.Print("Previews Flame at lvl52")
    Sol.io.Print("|cffffffff/skill " .. spellLink .. "|r")
    Sol.io.Print("Previews Flame at lvl50, assuming you linked flame from your skill book")
end


function ItemPreview_OnUpdate()
	if not isWorking then return end
	
	local self = ItemPreview
	
	self.frames.tooltip:SetText("")
	
	if ip_debug == true then Sol.io.Print(plusData) end
	
	local plusDataTemp = plusData:gsub("|Hitem:","")
	
	if ip_debug == true then Sol.io.Print(plusDataTemp) end
	
	local hash = CalculateItemLinkHash(plusDataTemp)
	
	local link = plusData .. string.format("%x", hash) .. plusName
	
	self.frames.tooltip:SetHyperLink(link)
	if _G[self.frames.tooltip:GetName() .. "TextLeft1"]:GetText() ~= "" then
		self:ShowLink(link, true)
--		if not IsShiftKeyDown() then
--			self:PrintLink(link, true)
--		end
		isWorking = nil
		self.frames.lblPlusWorking:SetText("")
		return
	else
		isWorking = nil
		Sol.io.Print("Not found")
		self.frames.lblPlusWorking:SetText("Not found")
		self.frames.tooltip:Hide()
		return
	end
end

--//////////////////////////////////////////////////////////////////

-- Runes of Magic item link hash calculation code

-- Author: Valacar (aka Duppy of the Runes of Magic US Osha server)
-- Release Date: September 12th, 2010

-- Credit goes to Neil Richardson for the xor, lshift, and rshift function
-- which I slightly modified. The original code can be found at:
-- http://luamemcached.googlecode.com/svn/trunk/CRC32.lua

-- I could care less what anyone does with the code (i.e. it's public domain),
-- but I'd very much appreciate being given credit (to me Valacar) if you do
-- use the code in any way.


-- Exclusive OR
local function xor(a, b)
    local calc = 0
    for i = 32, 0, -1 do
        local val = 2 ^ i
        local aa = false
        local bb = false
        if a == 0 then
            calc = calc + b
            break
        end
        if b == 0 then
            calc = calc + a
            break
        end
        if a >= val then
            aa = true
            a = a - val
        end
        if b >= val then
            bb = true
            b = b - val
        end
        if not (aa and bb) and (aa or bb) then
             calc = calc + val
        end
    end
     return calc
end

-- binary shift left
local function lshift(num, left)
     local res = num * (2 ^ left)
     return res % (2 ^ 32)
end

-- binary shift right
local function rshift(num, right)
    right = right % 0x20
    local res = num / (2 ^ right)
    return math.floor(res)
end

-- get lower word of a 32-bit number
local function loword(num)
    return rshift(lshift(num, 16), 16)
end

-- get high word of a 32-bit number
local function hiword(num)
    return rshift(num, 16) % 2^16
end

-- multiply two 32-bit numbers, but returns only the low dword of the 64-bit result
local function mymul(num1, num2)
    local x = loword(num2) * num1
    local y = (hiword(num2) * num1) * 2^16
    local a = hiword(x) + hiword(y)
    local b = loword(x)

    return (a * 2^16) + b
end

--//////////////////////////////////////////////////////////////////

-- Calculates hash value of an item based on first 11 hex numbers of an item link
function CalculateItemLinkHash(values)

	if ip_debug == true then Sol.io.Print(values) end

    assert(type(values) == "string", "Must pass a string to CalculateItemLinkHash consisting of 11 hex values separated by a space.")

    local num = { values:match("^(%x+)%s+(%x+)%s+(%x+)%s+(%x+)%s+(%x+)%s+(%x+)%s+(%x+)%s+(%x+)%s+(%x+)%s+(%x+)%s+(%x+)") }
    assert(#num == 11, "11 hex values required!")

    local sum = 0
    for s = 1, 11 do
        num[s] = tonumber(num[s], 16)
        sum = sum + num[s]
    end

    local a,b,c,d,e,x,i = sum,0,0,0,0,0,0

    repeat
        d = num[x+1]
        b = d
        b = b * x
        b = b % 2^32
        e = d
        e = rshift(e, i)
        i = i + 0x10
        x = x + 1
        e = e + a
        e = e % 2^32
        b = b + e
        b = b % 2^32
        b = xor(b, d)
        a = b
    until (i >= 0xB0)

    local j = 0

    repeat
        d = num[j+1]
        c = d + 1
        c = mymul(c, a)
        c = c % 2^32
        a = d
        a = mymul(a, c)
        a = a % 2^32
        a = rshift(a, 16)
        a = a + c
        j = j + 1
        a = xor(a, d)
    until (j >= 0x0B)

    hash = loword(a)

    return hash
end
